//Model Rocket Motor Thrust Test Dynamometer
//Created by NOSS, January 1 2021
//Credit to others that provided the bones of the sketch by their examples
//Please use and enjoy
//Please give credit
//Uses strain gauge to measure force applied over an 8 to 10 second period
//Records strain gauge readings directly onto an SD card in a CSV file
//Allows for safety interlock and time delayed triggering
//Checks status of SD card before allowing motor ignition
//SD card checked for presence, format and existence of earlier data files
//A new filename will be created for each test by incrementing the most recent filename by 1
//eg - last filename was DYNO_05, new filename will be DYNO_06
//This is to prevent accidental over-write of important data files.
//
//On power-up the message - "MODEL ROCKET DYNOMOMETER" is displayed for a few seconds
//The display then changes to "PREP SD CARD, SELECT TO FIRE"
//The system is triggered by pressing the select button on the LCD Shield
//The display changes to "INIT SD CARD"
//If the card is missing or faulty the message changes to "INIT FAILED"
//If SD card is OK the display shows "INIT SUCCESS"
//After a 2 second delay the display shows "STAND CLEAR, ACTIVATED"
//After a 3 second delay the countdown begins from 10. The display shows each number
//and there is a audio beep.
//When the countdown reaches 0, there is a long beeeeeeeep.
//The next sequences involve initialising the scale and igniting the motor.
//The motor is ignited by an electronic match powered by a relay driven by the Arduino
//The scale initialisation is carried out to get an average zero
//this is done in the time between the long beeeeeep and ignition of the motor.
//The motor ignite signal is sent, followed by a timestamp then a continuous recording of scale data
//onto the SD card into the previously calculated filename.
//The scale is modified to record 80 readings per second
//The system records 800 readings onto the SD card
//The last scale reading is followed by another timestamp.
//Writing the scale readings onto the SD card is slightly faster than sending
//the reading to the remote terminal.
//Consequently the 800 readings are recorded over about 9 seconds
//This makes the time axis on the graph slightly inaccurate but not too much
//At the end of the recording sequence the SD file is closed and the ingnition relay powered down.
//There is a return command at the end of the recording sequence so the system reverts to it's ready state
//waiting for a button to be pressed.


//The LCD shield demands digital IO pins 4, 5, 6, 7, 8 and 9. These cannot be changed.

#include <LiquidCrystal.h>
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);


#include "HX711.h"
#include <SPI.h>
#include <SD.h>


// start sequence button
//The buttonPin is for triggering away from the board by a remote switch.
//Because most of the digital IO has been used by the LCD, SD Card Shield and scale
//an Analogue IO has been used as a Digital IO

const int buttonPin = 15;

//As above, due to the lack of Digital IO, some Analogue IO is being used as Digital
//so A0 effectively becomes D14, A1-D15, A2-D16, A3-D17, A4-D18, A5-D19
//This is about as far as an Arduino UNO can be pushed

const int ledPin = 16;    //Used for the buzzer
const int igniterPin = 17 ;    //igniter output


//This sets the Chip Select pin for the SD card shield
//CS is the only pin on the SD card shield that can be configured.

const uint8_t CS_PIN = 2;;

int buttonState = 0;
int counter = 0;
int mytime = 0;
int incomingByte = 0;
int analogPin = A0;
//int buzzer = A5;
int buzzer = 23;
int Val = 0;
int chipset = 0;
int cd = 10;

//Pins for the scale input. 18 is the analogue pin A4 which has been repurposed for this project

#define DT 3
#define SCK 18


//The letters part of the filename to be written to the SD card. NOTE: the max number of characters for this part of the filename is 6
//This is because there will be a unique serial number added to rdyno - like rdyno099 for example.  That makes each test a unique file.

#define Launch
#define FILE_BASE_NAME "rdyno"   //This can be changed to whatever you want up to 6 characters


//The calibration factor is something that has to be experimentally determined. I used a 2 litre jug of water which weighed exactly 2kg as a test load.
//When the scale was triggered I worked out by poke and hope, what the offset should be.  This number is accurate for my system only.
//You will need to calibrate your own system.

HX711 scale;
float calibration_factor = -214.7;

File file;


const uint8_t BASE_NAME_SIZE = sizeof(FILE_BASE_NAME) - 1;
char fileName[] = FILE_BASE_NAME "00.csv";


//This is where the sketch returns after completion.  This is like the loop function but covers the whole setup onwards.  It's a much more efficient reset.

void(* resetFunc) (void) = 0;


//Sets pin modes

void setup() {
  pinMode(buttonPin, INPUT);
  pinMode(igniterPin, OUTPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(CS_PIN, OUTPUT);
  pinMode(buzzer, OUTPUT);




  Serial.begin(9600);
  lcd.begin(16, 2);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("  MODEL ROCKET");
  lcd.setCursor(0, 1);
  lcd.print("  DYNAMOMETER");
  delay(2000);


  //Gets an average zero (TARE) reading for the scale

  scale.begin(DT, SCK);
  scale.tare(); //Reset the scale to 0
  long zero_factor = scale.read_average(); //Get a baseline reading

}

void loop() {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(" PREP SD CARD");
  lcd.setCursor(0, 1);
  lcd.print(" SELECT TO FIRE");
  scale.set_scale(calibration_factor);
  delay(500);

  //The buttons on the LCD shield produce a single analogue value when pressed
  //Only one analogue input is used and each button creates a different analogue value
  //The select button generates an analogue value between 500 and 750

  buttonState = digitalRead(buttonPin);
  Val = analogRead (analogPin);

  //This line checks the button on the LCD shield and also the remote button input.
  //If either of these is TRUE then the initialisation process begins.
  //NOTE: It's important to have a pull-down resistor on the button input
  //to avoid false triggering.

  if (Val >= 500 && Val <= 750 || buttonState == HIGH ) {
    
    //Generate a short tone to indicate button has been pressed,
    tone(A5, 2000, 20);


    lcd.clear();
    lcd.setCursor(0, 0);

    // Initialise SD Card

    lcd.print(" INIT SD CARD");


    //Checks the status of the SD card and if no errors found passes onto the next step
    
    if (!SD.begin(2)) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print(" INIT FAILED");
      delay(5000);
      return;
    }


    //The following lines check the SD card for the latest data file name
    //The code checks the SD card to see if filename rdyno00 exists
    //if it does then 1 is added and the code checks for Dyno01
    //and so on until it cant find the next filename.
    //That number is used to create the new filename.
    //If for some reason the new filename cannot be created
    //The LCD shows "FILENAME ERROR" for 5 seconds then returns.
    //Only file numbers up to 99 can be used.  Check the SD card regularly
    //to archive old files.  After archiving I'd suggest changing the
    //base filename to make sure you don't end up with lots of rdyno01 files.

    while (SD.exists(fileName)) {
      if (fileName[BASE_NAME_SIZE + 1] != '9') {
        fileName[BASE_NAME_SIZE + 1]++;
      } else if (fileName[BASE_NAME_SIZE] != '9') {
        fileName[BASE_NAME_SIZE + 1] = '0';
        fileName[BASE_NAME_SIZE]++;
      } else {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print(" FILENAME ERROR");
        delay(5000);
        return;
      }
    }
    // If the SD Card checks and filename creation are good the the LCD shows "INIT SUCCESS"
    
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(" INIT SUCCESS");
    delay(2000);

    //After a couple of seconds the LCD changes to "STAND CLEAR, ACTIVATED" then the countdown begins

    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(" STAND CLEAR");
    lcd.setCursor(0, 1);
    lcd.print(" ACTIVATED");
    delay(3000);

    //This little section of code make a short beep on the buzzer

    tone(A5, 2000, 100);


    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("10");
    delay (1000);


    //This section writes "10" onto the LCD then waits for 1 second

    for (int cd = 10; cd >= 1; cd--) {
      tone(A5, 2000, 100);
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print(cd);
      delay (1000);
    }


    //Long beeeeep of buzzer to indicate countdown finished and stuff's about to happen

    tone(A5, 2000);
    delay(1500);
    noTone(A5);


    //Sets the scale calibration and shows "Writing to SD" on LCD

    scale.set_scale(calibration_factor); //Adjust to the calibration factor
    delay (1000); //I put this delay in for some reason. I don't know if it's necessary but it doesn't do any harm
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(" Writing to SD");


    // Energises the igniter relay

    digitalWrite(igniterPin, HIGH);


    //Opens the data file on the SD card with the previously determined filename.
    //Writes the start time by means of the millis (current time in milliseconds since sketch started)

    File dataFile = SD.open(fileName, FILE_WRITE);
    dataFile.println(millis());
    dataFile.println();

    //Adjust to the calibration factor

    scale.set_scale(calibration_factor);

    //Set loop for 800 samples at 80sa/sec = 10 seconds theoretical
    //The actual time is around 8.9 seconds to record 800 samples

    for (int i = 0; i <= 800; i++) {

      //Write the scale data to the SD card for 800 cycles.

      dataFile.println(scale.get_units(), 1);

    }

    //Create a blank line between the data and the time code

    dataFile.println();

    //Records the end of run time

    dataFile.println(millis());

    //Close the data file to ensure all data preserved

    dataFile.close();

    //Indicate that SD writing is donr, close the igniter pin, turn off the LED then return to standby.
    
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(" DONE WRITING");
    digitalWrite (ledPin, LOW);
    digitalWrite (igniterPin, LOW);
    delay(2000);
    resetFunc();
  }

  //End of sketch.
}
//Copyright (C) 2021 - Nightmare On Scam Street - nightmare.on.scam.street@gmail.com
